@@ -97,13 +97,15 @@ class TrainManager:
         self.early_stopping_metric = train_config.get("early_stopping_metric",
                                                       "eval_metric")
 
-        # if we schedule after BLEU/chrf, we want to maximize it, else minimize
         # early_stopping_metric decides on how to find the early stopping point:
-        # ckpts are written when there's a new high/low score for this metric
+        # ckpts are written when there's a new high/low score for this metric.
+        # If we schedule after BLEU/chrf/accuracy, we want to maximize the
+        # score, else we want to minimize it.
         if self.early_stopping_metric in ["ppl", "loss"]:
             self.minimize_metric = True
         elif self.early_stopping_metric == "eval_metric":
-            if self.eval_metric in ["bleu", "chrf"]:
+            if self.eval_metric in ["bleu", "chrf",
+                                    "token_accuracy", "sequence_accuracy"]:
                 self.minimize_metric = False
             # eval metric that has to get minimized (not yet implemented)
             else:
@@ -298,6 +300,7 @@ class TrainManager:
             total_valid_duration = 0
             start_tokens = self.total_tokens
             self.current_batch_multiplier = self.batch_multiplier
+            self.optimizer.zero_grad()
             count = self.current_batch_multiplier - 1
             epoch_loss = 0
 
@@ -314,11 +317,13 @@ class TrainManager:
 
                 # Set current_batch_mutliplier to fit
                 # number of leftover examples for last batch in epoch
-                if self.batch_multiplier > 1 and i == len(train_iter) - \
-                        math.ceil(leftover_batch_size / self.batch_size):
-                    self.current_batch_multiplier = math.ceil(
-                        leftover_batch_size / self.batch_size)
-                    count = self.current_batch_multiplier - 1
+                # Only works if batch_type == sentence
+                if self.batch_type == "sentence":
+                    if self.batch_multiplier > 1 and i == len(train_iter) - \
+                            math.ceil(leftover_batch_size / self.batch_size):
+                        self.current_batch_multiplier = math.ceil(
+                            leftover_batch_size / self.batch_size)
+                        count = self.current_batch_multiplier - 1
 
                 update = count == 0
                 # print(count, update, self.steps)
@@ -372,7 +377,8 @@ class TrainManager:
                             max_output_length=self.max_output_length,
                             loss_function=self.loss,
                             beam_size=1,  # greedy validations
-                            batch_type=self.eval_batch_type
+                            batch_type=self.eval_batch_type,
+                            postprocess=True # always remove BPE for validation
                         )
 
                     self.tb_writer.add_scalar("valid/valid_loss",
